package Hawat::Model::Alerts;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use HTML::Entities;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);
use Value::Convertor;
use Mentat::Message::Storage::Mongo;
use Mentat::Message::Convertor::Mongo;
use Mentat::Message::Query::Mongo;
use Data::Filter::Factory;
use Mentat::Storage::Mongo::Aggregator;

use Mentat::Message::Formater::Helper;
use Mentat::Message::Formater::TextTemplate;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

# Define meaningful default configuration values
use constant DFLT_FLAG_DEVEL => 0;

use constant UNDISCLOSED => "Undisclosed";
use constant MODEL_QUERY => "Query";

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class            => 'Hawat::Model::Alerts',
    constructor      => 'new',
    devel            => 0,
    database_name    => 'mentat',
    collection_name  => 'alerts',
    sdatabase_name   => 'mentat_stats',
    scollection_name => 'statistics',
    filter_grammar   => '/etc/mentat/.core/filter-grammar.conf',
    template_dir     => '/etc/mentat/formater-templates',
    cache_dir        => '/var/mentat/cache',
    cache_ttl        => 60*60*24,
    svc_helper       => '/etc/mentat/.core/svc.mentat-message-formater-helper.json',
);
has "devel"            => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"    => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name"  => (is => 'ro', isa => 'Str',  required => 1);
has "sdatabase_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "scollection_name" => (is => 'ro', isa => 'Str',  required => 1);
has "filter_grammar"   => (is => 'ro', isa => 'Str',  required => 1);
has "template_dir"     => (is => 'ro', isa => 'Str',  required => 1);
has "cache_dir"        => (is => 'ro', isa => 'Str',  required => 1);
has "cache_ttl"        => (is => 'ro', isa => 'Int',  required => 1);
has "db"               => (is => 'rw', isa => 'Mentat::Message::Storage::Mongo');
has "query"            => (is => 'rw', isa => 'Mentat::Message::Query::Mongo');
has "aggregator"       => (is => 'rw', isa => 'Mentat::Storage::Mongo::Aggregator');
has "ffactory"         => (is => 'rw', isa => 'Data::Filter::Factory');
has "helper"           => (is => 'rw', isa => 'Mentat::Message::Formater::Helper');
has "frme"             => (is => 'rw', isa => 'Mentat::Message::Formater::TextTemplate');
has "frma"             => (is => 'rw', isa => 'Mentat::Message::Formater::TextTemplate');
has "svc_helper"       => (is => 'ro', isa => 'Str',  required => 1);

# Validation hashes
my %SORTBYS = (
        'ts' => '_CESNET.StorageTime',
        'dt' => 'DetectTime',
        'ct' => 'CreateTime',
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc' => 1,
        'descending' => -1,
        'ascending' => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

sub get_item_sets($;$$)
{
    my $self = shift;
    my ($rewrite, $stats_only) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->aggregator->get_item_sets($rewrite, $stats_only);
}

#-------------------------------------------------------------------------------
# Interface required by Hawat::Controller::DBItem
#-------------------------------------------------------------------------------

# find($$$$$$)
#
#  Usage     : my ($alerts, $count, $count_all, $duration, $query) = $c->model('Alerts')->find($search, $limit, $skip, $sortby, $sort, $quick);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick, $c) = @_;

    # Initialize the sorting specification
    my ($sort_spec, $sort_spec_j);
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec   = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
        $sort_spec_j = '{"'.$SORTBYS{$sortby}.'":'.$SORTTYPES{$sort}.'}';
    }

    my $start = time();
    my ($alerts, $count_all);
    if (defined $search->{raw_filter} && length $search->{raw_filter} > 0) {
        if ($search->{raw_filter} =~ /^{/) {
            ($alerts, $count_all) = $self->query->query_msg_json($search->{raw_filter}, $sort_spec_j, $limit, $skip, $quick);
        }
        else {
            Log::Loger->warn(__PACKAGE__, "[HMCA] PARSING FILTER '$search->{raw_filter}'");
            my $filter_tree = $self->ffactory->filter_from_string($search->{raw_filter});
            Log::Loger->warn(__PACKAGE__, "[HMCA] DONE PARSING FILTER '$search->{raw_filter}'");
            #$c->log->debug("Filter tree: ".Dumper($filter_tree));
            #$c->log->debug("Filter query: ".Dumper($self->query->generate_query($filter_tree)));
            Log::Loger->warn(__PACKAGE__, "[HMCA] STARTING DATABASE QUERY");
            ($alerts, $count_all) = $self->query->query_msg($filter_tree, $sort_spec, $limit, $skip, $quick);
        }
    }
    else {
        Log::Loger->warn(__PACKAGE__, "[HMCA] STARTING FIND ALL QUERY");
        ($alerts, $count_all) = $self->db->find_msg(undef, $sort_spec, $limit, $skip, $quick);
    }
    my $count = scalar @$alerts;
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, $count_all, $duration, $self->db->get_last_filter()) unless $count;
    return ($alerts, $count, $count_all, $duration, $self->db->get_last_filter(), $self->db->get_last_index(1));
}

# fetch($)
#
#  Usage     : my ($report, $count, $duration) = $c->model('Alerts')->fetch($id);
#  Purpose   : Get one particular alert within the alert storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($alert_id,) = @_;
    die "Alert ID must be given as argument\n" unless $alert_id;

    my $start = time();
    my ($alert, $raw_alert) = $self->db->fetch_msg_wr($alert_id);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $alert;

    return ($alert, 1, $duration, $raw_alert);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Alerts')->delete($id);
#  Purpose   : Delete particular alert from the alert storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($alert_id,) = @_;
    die "Alert ID must be given as argument\n" unless $alert_id;

    return $self->db->delete($alert_id);
}

# generate_email($)
#
#  Usage     : my $rv = $c->model('Alerts')->generate_email($message);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub generate_email($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;

    my @section_ids = $self->frme->detect_section_ids($message);

    my $result = {};
    foreach my $type ('summary', 'extra') {
        $result->{$type}->{email} = $self->frme->format_section($section_ids[0], [$message], $type, 'default');
        $result->{$type}->{attch} = $self->frma->format_section($section_ids[0], [$message], $type, 'default');
    }
    return $result;
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal helper objects for whole model
    $self->db(Mentat::Message::Storage::Mongo->new(
        database   => $self->database_name,
        collection => $self->collection_name
    ));
    $self->ffactory(Data::Filter::Factory->new_from_file(
        $self->filter_grammar,
    ));
    $self->query(Mentat::Message::Query::Mongo->new(
        convertor => Mentat::Message::Convertor::Mongo->new(),
        storage   => $self->db,
    ));
    $self->aggregator(Mentat::Storage::Mongo::Aggregator->new(
        database    => $self->database_name,
        collection  => $self->collection_name,
        sdatabase   => $self->sdatabase_name,
        scollection => $self->scollection_name,
        cache       => $self->cache_dir,
        ttl         => $self->cache_ttl,
    ));
    $self->helper(Mentat::Message::Formater::Helper->service($self->svc_helper));
    $self->frme(Mentat::Message::Formater::TextTemplate->new(
        $self->template_dir . '/email',
        $self->helper,
    ));
    $self->frma(Mentat::Message::Formater::TextTemplate->new(
        $self->template_dir . '/csv',
        $self->helper,
    ));

    # If you use 'weak_ref' for moose attribute, previous calls will not set the
    # attributes. It is necessary to do it with antermediate variables $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
    die "Unable to initialize filter factory attribute\n"      unless $self->ffactory;
    die "Unable to initialize database query attribute\n"      unless $self->query;
    die "Unable to initialize database aggregator attribute\n" unless $self->aggregator;
    die "Unable to initialize formater helper attribute\n"     unless $self->helper;
    die "Unable to initialize email formater attribute\n"      unless $self->frme;
    die "Unable to initialize attachment formater attribute\n" unless $self->frma;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
