package Hawat::Model::Query;
use strict;
use warnings;
use base 'Catalyst::Model::Adaptor';

__PACKAGE__->config(
    class       => 'Hawat::Model::Query',
    constructor => 'new',
);

use Carp;
use Scalar::Util 'blessed';
use HTML::Entities;
use Mentat::Core qw(:c_hawat);

use constant ID_QUERY => "_id";

# Define meaningful default configuration values
use constant DFLT_FLAG_DEVEL    => 0;

# Load global configurations from config file
my $CFG_G = Hawat->config;

# Load module specific configurations from config file
my $CFG = $CFG_G->{"Model::Query"};

# Setup global configurations
my $DEVEL = (exists($CFG_G->{devel})) ? $CFG_G->{devel} : DFLT_FLAG_DEVEL();

# Overwrite with module specific configurations
$DEVEL = $CFG->{devel} if (exists($CFG->{devel}));
if ($DEVEL) {
    use Data::Dumper;
}

sub clean($)
{
    my $text = shift;
    
}

sub process_form($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form) = @_;

    # Copy the identifier into another attribute
    $form->{_id} = $form->{query_name};

    # Separate form arguments (only for readability in follwing code)
    my $advanced = $form->{advanced};
    my $ipsrc    = $form->{source};
    my $logop    = $form->{logop} || Mentat::Core::OP_AND;
    my $iptgt    = $form->{target};
    my $from     = $form->{datefrom};
    my $to       = $form->{dateto};
    my $detector = $form->{detector};
    my $category = $form->{category};
    my $abuse    = $form->{abuse};

    # Do not do any alterations on advanced form
    return $form if $advanced;

    # Query arguments
    my @query;

    my ($ips_tmp, $ipd_tmp, $op);

    # Process source and destination related filter rules
    if (defined $ipsrc && length($ipsrc) > 0 && defined $iptgt && length($iptgt)) {
        $ips_tmp = HAWAT_PATH_SOURCE_IP().' '.Mentat::Core::OP_EQ.' "'.$ipsrc.'"';
        $ipd_tmp = HAWAT_PATH_TARGET_IP().' '.Mentat::Core::OP_EQ.' "'.$iptgt.'"';
        $op      = (uc($logop) eq Mentat::Core::OP_OR) ? Mentat::Core::OP_P_OR() : Mentat::Core::OP_P_AND();

        push (@query, "$ips_tmp $op $ipd_tmp");
    }
    else {
        if (defined $ipsrc && length($ipsrc) > 0) {
            $ips_tmp = HAWAT_PATH_SOURCE_IP().' '.Mentat::Core::OP_EQ().' "'.$ipsrc.'"';

            push (@query, "$ips_tmp");
        }
        if (defined $iptgt && length($iptgt) > 0) {
            $ipd_tmp = HAWAT_PATH_TARGET_IP().' '.Mentat::Core::OP_EQ().' "'.$iptgt.'"';

            push (@query, "$ipd_tmp");
        }
    }

    # Process time related filter rules
    if (defined $from && length($from) > 0) {
        push (@query, "DetectTime ".Mentat::Core::OP_GE().' "'.$from.'"');
    }
    if (defined $to && length($to) > 0) {
        push (@query, "DetectTime ".Mentat::Core::OP_LE().' "'.$to.'"');
    }

    # Process detector related filter rules
    if (defined $detector)  {
        # Detector is multiselect, so make sure we work with array
        $detector = [$detector] unless ref $detector eq 'ARRAY';
        my %h = map { $_, 1 } @$detector;
        push (@query, 'Node/Name '.Mentat::Core::OP_IN.' ["'.join('","', (sort keys %h)).'"]');
    }

    # Process category related filter rules
    if (defined $category) {
        # Category is multiselect, so make sure we work with array
        $category = [$category] unless ref $category eq 'ARRAY';
        my %h = map { $_, 1 } @$category;
        push (@query, 'Category '.Mentat::Core::OP_IN.' ["'.join('","', (sort keys %h)).'"]');
    }

    # Process abuse related filter rules
    if (defined $abuse) {
        # Abuse is multiselect, so make sure we work with array
        $abuse = [$abuse] unless ref $abuse eq 'ARRAY';
        my %h = map { $_, 1 } @$abuse;
        push (@query, '_CESNET/ResolvedAbuses '.Mentat::Core::OP_IN.' ["'.join('","', (sort keys %h)).'"]');
    }
    
    # Build all the chunks of the query together
    $form->{raw_filter} = (@query <= 1) ? "@query" : (''.join(' '.Mentat::Core::OP_AND.' ', @query));

    Log::Loger->info(__PACKAGE__, "Translated alerts form filter: $form->{raw_filter}") if ($DEVEL);
    return $form;
}

sub getSelected($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($data) = @_;

    unless (defined $data) {
        return undef;
    }
    elsif (ref($data) eq 'ARRAY') {
        my %hash = map { $_ => "selected" } @$data;
        return \%hash;
    }
    else {
        return { $data => "selected" };
    }
}

sub getSelector($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($data) = @_;

    if (! defined $data) {
        return "";
    }
    elsif (ref($data) eq 'ARRAY') {
        return join( ';', @$data );
    }
    else
    {
        return $data;
    }
}

1;
